==> pagerer-8.x-1.0-alpha1.tar.gz: pagerer/src/Pagerer.php <==
<?php

/**
 * @file
 * Contains \Drupal\pagerer\Pagerer.
 */

namespace Drupal\pagerer;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Pagerer pager management class.
 */
class Pagerer implements ContainerInjectionInterface {

  /**
   * The pager element.
   *
   * This is the index used by query extenders to identify the query
   * to be paged, and reflected in the 'page=x,y,z' query parameter
   * of the HTTP request.
   *
   * @var int
   */
  protected $element;

  /**
   * The route name.
   *
   * @var string
   */
  protected $routeName;

  /**
   * The pager adaptive keys.
   *
   * @var string
   */
  protected $adaptiveKeys;

  /**
   * The Pagerer factory.
   *
   * @var \Drupal\pagerer\PagererFactory
   */
  protected $factory;

  /**
   * The request stack object.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The URL generator.
   *
   * @var \Drupal\Core\Routing\UrlGeneratorInterface
   */
  protected $urlGenerator;

  /**
   * Constructs a new Pagerer pager object.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Routing\UrlGeneratorInterface $url_generator
   *   The URL generator.
   */
  public function __construct(PagererFactory $pagerer_factory, RequestStack $request_stack, UrlGeneratorInterface $url_generator) {
    $this->factory = $pagerer_factory;
    $this->requestStack = $request_stack;
    $this->urlGenerator = $url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $element = NULL) {
    $instance = new static(
      $container->get('pagerer.factory'),
      $container->get('request_stack'),
      $container->get('url_generator')
    );

    // Set the pager element.
    $instance->element = $element;

    // Set the pager adaptive keys if they exist in the query string.
    if ($page_ak = $instance->getCurrentRequest()->query->get('page_ak')) {
      // A 'page_ak' query parameter exists in the calling URL.
      $adaptive_keys = explode(',', $page_ak);
      if (isset($adaptive_keys[$element])) {
        $instance->setAdaptiveKeys($adaptive_keys[$element]);
      }
    }

    return $instance;
  }

  /**
   * Gets the route name for this pager.
   *
   * @return string
   *   The route name.
   */
  public function getRouteName() {
    return $this->routeName;
  }

  /**
   * Sets the route name for this pager.
   *
   * @param string $page
   *   The route name.
   *
   * @return $this
   */
  public function setRouteName($route_name) {
    $this->routeName = $route_name;
    return $this;
  }

  /**
   * Gets the pager element.
   *
   * @return int
   *   The pager element.
   */
  public function getElement() {
    return $this->element;
  }

  /**
   * Returns the current request object.
   *
   * @return \Symfony\Component\HttpFoundation\Request
   *   The request object.
   */
  protected function getCurrentRequest() {
    return $this->requestStack->getCurrentRequest();
  }

  /**
   * Initializes the pager.
   *
   * @param int $total
   *   The total number of items to be paged.
   * @param int $limit
   *   The number of items the calling code will display per page.
   *
   * @return \Drupal\pagerer\Pagerer
   *   The Pagerer pager object.
   */
  public function init($total, $limit) {
    pager_default_initialize($total, $limit, $this->element);
    return $this;
  }

  /**
   * Gets the current page.
   *
   * @return int
   *   The page to which the pager is currently positioned to.
   */
  public function getCurrentPage() {
    global $pager_page_array;
    return isset($pager_page_array[$this->element]) ? $pager_page_array[$this->element] : NULL;
  }

  /**
   * Sets the current page for this pager.
   *
   * @param int $page
   *   The page to which the pager is currently positioned to.
   *
   * @return \Drupal\pagerer\Pagerer
   *   The Pagerer pager object.
   */
  public function setCurrentPage($page) {
    global $pager_page_array;
    $pager_page_array[$this->element] = $page;
    return $this;
  }

  /**
   * Gets total pages in the pager.
   *
   * @return int
   *   The total number of pages managed by the pager.
   */
  public function getTotalPages() {
    global $pager_total;
    return isset($pager_total[$this->element]) ? $pager_total[$this->element] : 0;
  }

  /**
   * Gets last page in the pager (zero-index).
   *
   * @return int
   *   The index of the last page in the pager.
   */
  public function getLastPage() {
    global $pager_total;
    return isset($pager_total[$this->element]) ? $pager_total[$this->element] - 1 : -1;
  }

  /**
   * Gets total items in the pager.
   *
   * @return int
   *   The total number of items (records) managed by the pager.
   */
  public function getTotalItems() {
    global $pager_total_items;
    return isset($pager_total_items[$this->element]) ? $pager_total_items[$this->element] : 0;
  }

  /**
   * Gets the items per page.
   *
   * @return int
   *   The number of items (records) in each page.
   */
  public function getLimit() {
    global $pager_limits;
    return isset($pager_limits[$this->element]) ? $pager_limits[$this->element] : 0;
  }

  /**
   * Gets the adaptive keys of this pager.
   *
   * Used by the Adaptive pager style.
   *
   * @return string
   *   The adaptive keys string, in the format 'L.R.X', where L is the
   *   adaptive lock to left page, R is the adaptive lock to right page,
   *   and X is the adaptive center lock for calculation of neighborhood.
   */
  public function getAdaptiveKeys() {
    return $this->adaptiveKeys;
  }

  /**
   * Sets the adaptive keys of this pager.
   *
   * Used by the Adaptive pager style.
   *
   * @param string $adaptive_keys
   *   The adaptive keys string, in the format 'L.R.X', where L is the
   *   adaptive lock to left page, R is the adaptive lock to right page,
   *   and X is the adaptive center lock for calculation of neighborhood.
   *
   * @return \Drupal\pagerer\Pagerer
   *   The Pagerer pager object.
   */
  protected function setAdaptiveKeys($adaptive_keys) {
    $this->adaptiveKeys = $adaptive_keys;
    return $this;
  }

  /**
   * Gets the query parameters array of a pager link.
   *
   * @param array $parameters
   *   An associative array of query string parameters to append to the pager
   *   links.
   * @param int $page
   *   The target page.
   * @param string $adaptive_keys
   *   The adaptive keys string, in the format 'L.R.X', where L is the
   *   adaptive lock to left page, R is the adaptive lock to right page,
   *   and X is the adaptive center lock for calculation of neighborhood.
   *
   * @return array
   *   The updated array of query parameters.
   */
  public function getQueryParameters(array $parameters, $page, $adaptive_keys = NULL) {
    // Build the 'page' and 'page_ak' query parameter elements.
    // This is built based on the current page of each pager element (or NULL
    // if the pager is not set), with the exception of the requested page index
    // for the current element.
    $page_el = array();
    $page_ak = array();
    foreach ($this->factory->all() as $i => $p) {
      $page_el[$i] = ($i == $this->getElement()) ? $page : $p->getCurrentPage();
      $page_ak[$i] = ($i == $this->getElement()) ? $adaptive_keys : $p->getAdaptiveKeys();
    }

    // Build the 'page' and 'page_ak' fragments, removing unneeded trailing
    // keys.
    while (end($page_el) === NULL) {
      array_pop($page_el);
    }
    $parameters['page'] = implode(',', $page_el);
    while (end($page_ak) === NULL) {
      array_pop($page_ak);
    }
    $parameters['page_ak'] = implode(',', $page_ak);

    // Merge the updated pager query parameters, with any parameters coming
    // from the current request. In case of collision, current parameters
    // take precedence over the request ones.
    if ($current_request_query = $this->getCurrentRequest()->query->all()) {
      $parameters = array_merge($current_request_query, $parameters);
    }

    // Explicitly remove the segments if not relevant.
    if (empty($page_el)) {
      unset($parameters['page']);
    }
    if (empty($page_ak)) {
      unset($parameters['page_ak']);
    }

    return $parameters;
  }

  /**
   * Gets a pager link.
   *
   * @param array $parameters
   *   An associative array of query string parameters to append to the pager
   *   links.
   * @param int $page
   *   The target page.
   * @param string $adaptive_keys
   *   The adaptive keys string, in the format 'L.R.X', where L is the
   *   adaptive lock to left page, R is the adaptive lock to right page,
   *   and X is the adaptive center lock for calculation of neighborhood.
   * @param bool $set_query
   *   Whether the link should contain the query parameters.
   *
   * @return string
   *   The link.
   */
  public function getHref(array $parameters, $page, $adaptive_keys = NULL, $set_query = TRUE) {
    $options = $set_query ? ['query' => $this->getQueryParameters($parameters, $page, $adaptive_keys)] : [];
    return $this->urlGenerator->generateFromRoute($this->getRouteName(), [], $options);
  }

}

==> pagerer-8.x-2.0-alpha1.tar.gz: pagerer/src/Pagerer.php <==
<?php

namespace Drupal\pagerer;

use Drupal\Core\Pager\Pager;

/**
 * Pagerer pager value object.
 */
class Pagerer extends Pager {

  /**
   * The pager element.
   *
   * This is the index used by query extenders to identify the query
   * to be paged, and reflected in the 'page=x,y,z' query parameter
   * of the HTTP request.
   *
   * @var int
   */
  protected $element;

  /**
   * The route name.
   *
   * @var string
   */
  protected $routeName;

  /**
   * The route parameters.
   *
   * @var string[]
   */
  protected $routeParameters = [];

  /**
   * The pager adaptive keys.
   *
   * @var string
   */
  protected $adaptiveKeys;

  /**
   * Gets the route name for this pager.
   *
   * @return string
   *   The route name.
   */
  public function getRouteName() {
    return $this->routeName;
  }

  /**
   * Sets the route name for this pager.
   *
   * @param string $route_name
   *   The route name.
   *
   * @return $this
   */
  public function setRouteName($route_name) {
    $this->routeName = $route_name;
    return $this;
  }

  /**
   * Gets the route parameters for this pager.
   *
   * @return string[]
   *   The route parameters.
   */
  public function getRouteParameters() {
    return $this->routeParameters;
  }

  /**
   * Sets the route parameters for this pager.
   *
   * @param string[] $route_parameters
   *   The route parameters.
   *
   * @return $this
   */
  public function setRouteParameters(array $route_parameters) {
    $this->routeParameters = $route_parameters;
    return $this;
  }

  /**
   * Gets the pager element.
   *
   * @return int
   *   The pager element.
   */
  public function getElement() {
    return $this->element;
  }

  /**
   * Sets the pager element.
   *
   * @param int $element
   *   The pager element.
   *
   * @return $this
   */
  public function setElement($element) {
    $this->element = $element;
    return $this;
  }

  /**
   * Gets last page in the pager (zero-index).
   *
   * @return int
   *   The index of the last page in the pager.
   */
  public function getLastPage() {
    return $this->totalPages - 1;
  }

  /**
   * Gets the adaptive keys of this pager.
   *
   * Used by the Adaptive pager style.
   *
   * @return string
   *   The adaptive keys string, in the format 'L.R.X', where L is the
   *   adaptive lock to left page, R is the adaptive lock to right page,
   *   and X is the adaptive center lock for calculation of neighborhood.
   */
  public function getAdaptiveKeys() {
    return $this->adaptiveKeys;
  }

  /**
   * Sets the adaptive keys of this pager.
   *
   * Used by the Adaptive pager style.
   *
   * @param string $adaptive_keys
   *   The adaptive keys string, in the format 'L.R.X', where L is the
   *   adaptive lock to left page, R is the adaptive lock to right page,
   *   and X is the adaptive center lock for calculation of neighborhood.
   *
   * @return $this
   */
  public function setAdaptiveKeys($adaptive_keys) {
    $this->adaptiveKeys = $adaptive_keys;
    return $this;
  }

}
